"""A package that contains classes and functions to manage the dataset.
"""
from .dataset import Dataset
from .data_loc_info import DataLocInfo
from .input import Input
from .target import Target
from .test_data import TestData
from .train_data import TrainData
